//
// Font handling
// Fonts are fixed proportion, stored in bitmaps
//

#include "renderlib.h"
#include "surface.h"
#include "font.h"


// Return the width and height of a single font character
//
uint32_t EXPORT fontGetCharWidth(const Font* mFont)
{

	return mFont->charWidth;

}

uint32_t EXPORT fontGetCharHeight(const Font* mFont)
{

	return mFont->charHeight;

}


// Return the surface of a font
//
Surface* EXPORT fontGetSurface(const Font* mFont)
{

	return mFont->Surface;

}


// Load a font
//
Font* EXPORT fontLoad(const char* fileName)
{

	Font*	Fnt;


	Fnt = malloc(sizeof(Font));
	Fnt->Surface = surfaceReadFromPNG(fileName);

	if (Fnt->Surface)
	{
		Fnt->charWidth  = Fnt->Surface->Width / 128;
		Fnt->charHeight = Fnt->Surface->Height / 2;
	}
	else
	{
		free(Fnt);
		return NULL;
	}

	return Fnt;

}



// Destroy a font
//
void EXPORT fontDestroy(Font* Fnt)
{

	if (!Fnt)
		return;

	surfaceDestroy(Fnt->Surface);
	free(Fnt);

}