//
// Undo functions
// Keeps a linked list of unspecified data.
// Portions from http://gethelp.devx.com/techtips/cpp_pro/10min/10min0599.asp
//

#include "renderlib.h"
#include "undo.h"


undoNode_t*		pHead;
undoNode_t*		pTail;
uint32_t		maxNodes;
uint32_t		nodeCount;


// Remove a node from the list
//
static void undoRemoveNode(undoNode_t* pNode)
{

	if (pNode->pPrev == NULL)
		pHead = pNode->pNext;
	else
		pNode->pPrev->pNext = pNode->pNext;

	if (pNode->pNext == NULL)
		pTail = pNode->pPrev;
	else
		pNode->pNext->pPrev = pNode->pPrev;

	free(pNode->Data);
	free(pNode);

	nodeCount--;

}


// Add a new node
//
static void undoAddNode(undoNode_t* pNode)
{

	if (pHead == NULL)
	{
		pHead = pNode;
		pNode->pPrev = NULL;
	}
	else
	{
		pTail->pNext = pNode;
		pNode->pPrev = pTail;
	}
	pTail = pNode;
	pNode->pNext = NULL;

}


// Initialise undo
//
void EXPORT undoInit()
{

	pHead = NULL;
	pTail = NULL;
	nodeCount = 0;

}


// Set the maximum of undo levels
//
void EXPORT undoSetMax(const uint32_t maxUndo)
{

	// Shrink if needed
	while (nodeCount > maxUndo)
		undoRemoveNode(pHead);

	maxNodes = maxUndo;

}


// Destroy undo data
//
void EXPORT undoDestroy()
{

	while (pHead != NULL)
      undoRemoveNode(pHead);

}


// Perform an undo and remove a level
//
void EXPORT undoDoUndo(void* Data)
{

	if (nodeCount == 0)
		return;

	// Copy
	memcpy(Data, pTail->Data, pTail->dataLen);

	// Remove head
	undoRemoveNode(pTail);

}


// Add a new undo level
//
void EXPORT undoAdd(const void* Data, const uint32_t dataLen)
{

	undoNode_t*	pNode;


	// Setup new
	pNode = calloc(1, sizeof(undoNode_t));
	pNode->Data = malloc(dataLen);
	pNode->dataLen = dataLen;

	// Copy data
	memcpy(pNode->Data, Data, dataLen);

	// Add it
	undoAddNode(pNode);
	nodeCount++;

	// Remove tail if needed
	if (nodeCount > maxNodes)
	{
		undoRemoveNode(pHead);
		nodeCount--;
	}

}