VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsBackBuffer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Option Compare Binary
Option Base 0


' Bitmap object data
Private Type BitmapData
    bmType As Long
    bmWidth As Long
    bmHeight As Long
    bmWidthBytes As Long
    bmPlanes As Integer
    bmBitsPerPixel As Integer
    bmBitsPointer As Long
End Type

' BGRA palette item
Private Type rgbQuad
    rgbBlue As Byte
    rgbGreen As Byte
    rgbRed As Byte
    rgbAlpha As Byte
End Type

' Bitmap Info Header
Private Type bitmapInfoHeader
     biSize As Long
     biWidth As Long
     biHeight As Long
     biPlanes As Integer
     biBitCount As Integer
     biCompression As Long
     biSizeImage As Long
     biXPelsPerMeter As Long
     biYPelsPerMeter As Long
     biClrUsed As Long
     biClrImportant As Long
End Type

Private Type bitmapInfo
    biHeader As bitmapInfoHeader
    biColors(0 To 255) As rgbQuad
End Type


' Public properties
Public ptrSurface As Long
Public Width As Long
Public Height As Long
Public renderScale As Long

' Private properties
Private bmWidth As Long
Private bmHeight As Long

' Handles
Private hBitmap As Long
Private hDC As Long
Private bmpDataPointer As Long


' API calls
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hDC As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hDC As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hDC As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function CreateDIBSection Lib "gdi32" (ByVal hDC As Long, ByRef pBMI As bitmapInfo, ByVal iUsage As Long, ByRef ppvBits As Long, ByVal hSection As Long, ByVal dwOffset As Long) As Long
Private Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
Private Declare Function GetObject Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, ByRef lpObject As Any) As Long
Private Declare Sub FillMemory Lib "kernel32" Alias "RtlFillMemory" (Destination As Any, ByVal Length As Long, ByVal Fill As Byte)


Public Function writeToPNG(ByVal fileName As String, ByVal Compression As Byte) As Boolean

    If surfaceWriteToPNG(ptrSurface, fileName, Compression) = 1 Then Exit Function
    
    writeToPNG = True

End Function


' Clear the surface
Public Sub Clear()

    surfaceClear ptrSurface

End Sub


' Blit the surface onto a picturebox
Public Sub Present(ByVal hBoxDC As Long)

    If renderScale = 1 Then
        surfaceCopyToBitmap ptrSurface, bmpDataPointer
        BitBlt hBoxDC, 0, 0, Width, Height, hDC, 0, 0, vbSrcCopy
    
    ElseIf renderScale = 2 Then
        surfaceCopyToBitmapDouble ptrSurface, bmpDataPointer, bmWidth, bmHeight
        BitBlt hBoxDC, 0, 0, bmWidth, bmHeight, hDC, 0, 0, vbSrcCopy
        
    End If

End Sub


' Resize the backbuffer surface, remembering the scale
Public Sub Resize(ByVal nWidth As Long, ByVal nHeight As Long)

    Dim oldScale As Long
    
    
    oldScale = renderScale
    Reset
    Me.Create nWidth, nHeight, oldScale

End Sub


' Create a new backbuffer surface
Public Function Create(ByVal nWidth As Long, ByVal nHeight As Long, ByVal rScale As Long) As Boolean

    Dim tempBitmap As bitmapInfo
    Dim dataPointer As Long
    

    Reset
    
    
    ' Create device context
    hDC = CreateCompatibleDC(0)
    If hDC = 0 Then
        Exit Function
    End If
    
    
    ' Create a DIB
    With tempBitmap.biHeader
        .biSize = LenB(tempBitmap.biHeader)
        .biWidth = nWidth * rScale
        .biHeight = nHeight * rScale
        .biBitCount = 32
        .biPlanes = 1
        .biSizeImage = .biWidth * .biHeight * 4
        
        bmWidth = .biWidth
        bmHeight = .biHeight
    End With

    hBitmap = CreateDIBSection(hDC, tempBitmap, 0, dataPointer, 0, 0)
    If hBitmap = 0 Then
        DeleteDC hDC
        Exit Function
    End If
    
    FillMemory ByVal dataPointer, tempBitmap.biHeader.biSizeImage, 0
    
    
    ' Select DIB into device context
    If SelectObject(hDC, hBitmap) = 0 Then
        DeleteObject hBitmap
        DeleteDC hDC
        Exit Function
    End If
    
    ' Create a surface to match the DIB
    ptrSurface = surfaceCreate(nWidth, nHeight)
    If ptrSurface = 0 Then
        DeleteObject hBitmap
        DeleteDC hDC
        Exit Function
    End If
    
    
    bmpDataPointer = dataPointer
    Width = nWidth
    Height = nHeight
    renderScale = rScale
    
    Create = True

End Function


' Cleanup
Private Sub Reset()

    DeleteObject hBitmap
    DeleteDC hDC
    surfaceDestroy ptrSurface
    
    ptrSurface = 0
    Width = 0
    Height = 0
    bmWidth = 0
    bmHeight = 0
    renderScale = 1

End Sub


' Class
Private Sub Class_Terminate()

    Reset

End Sub
