VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsTileSet"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Option Compare Binary
Option Base 0


Private Tiles() As clsSurface
Private collisionTiles() As clsSurface
Public nTiles As Long

Private Collision() As Byte

Public pTileSize As Long


' Render tileset to a backbuffer
Public Sub Render(ByRef BackBuffer As clsBackBuffer)

    Dim X As Long, Y As Long
    Dim Idx As Long
    
    
    For Y = 0 To BackBuffer.Height / pTileSize
        For X = 0 To BackBuffer.Width / pTileSize
            renderBlitFast BackBuffer.ptrSurface, Tiles(Idx).ptrSurface, X * pTileSize, Y * pTileSize
            
            Idx = Idx + 1
            If Idx = nTiles Then Exit Sub
        Next X
    Next Y

End Sub


' Load tileset graphics from a bitstream
Public Function loadFromBits(ByRef Bits As clsBitStream, ByVal nTileSize As Long, ByVal Count As Long, ByRef Pal As clsPalette)

    Dim A As Long
    Dim BP As clsBitplane
    
    
    nTiles = Count
    ReDim Tiles(nTiles - 1)
    
    For A = 0 To nTiles - 1
        Set BP = New clsBitplane
        BP.loadPlanar Bits, nTileSize, nTileSize, 4
        Set Tiles(A) = BP.toSurface(Nothing, Pal.ptrPal, -1, 0)
    Next A
    
    pTileSize = nTileSize
    
End Function


' Return a tile's surface
Public Function Tile(ByVal Index As Long) As clsSurface

    Set Tile = Tiles(Index)

End Function

Public Function collisionTile(ByVal Index As Long) As clsSurface

    Set collisionTile = collisionTiles(Index)

End Function


' Load tile collision subtiles
Public Function loadCollisionFrom(ByVal Bits As clsBitStream)

    Dim A As Long, B As Long
    Dim X As Long, Y As Long
    
    Dim Idx As Long, Col As Long
    Dim Surface As clsSurface
    Dim noBlocks As Boolean
    
    Dim COL_DESTRUCT As Long
    Dim COL_SOLID As Long
    Dim COL_HURT As Long
    Dim COL_SECRET As Long
        
    
    COL_DESTRUCT = createBGRA(0, 255, 0, 0)
    COL_SOLID = createBGRA(255, 255, 255, 0)
    COL_HURT = createBGRA(255, 0, 0, 0)
    COL_SECRET = createBGRA(255, 0, 255, 0)
    
    
    ReDim Collision(nTiles - 1, 15)
    For A = 0 To nTiles - 1
        For B = 0 To 15
            Collision(A, B) = Bits.getByte
        Next B
    Next A
    
    ' Render a collision map for each tiles
    ReDim collisionTiles(nTiles - 1)
    For A = 0 To nTiles - 1
        Set Surface = New clsSurface
        Surface.Create pTileSize, pTileSize
        
        renderBlitFast Surface.ptrSurface, Tiles(A).ptrSurface, 0, 0
        
        noBlocks = True
        For Y = 0 To 3
            For X = 0 To 3
                Idx = Collision(A, Y * 4 + X)

                If Idx Then
                    Select Case Idx
                        ' Destructible
                        Case 127
                            Col = COL_DESTRUCT

                        ' Solid
                        Case 1
                            Col = COL_SOLID

                        ' Hurts
                        Case 211
                            Col = COL_HURT

                        ' "Secret"
                        Case 128
                            Col = COL_SECRET

                        Case Not 0
                            ' NOP
                            
                    End Select

                    renderBoxFill Surface.ptrSurface, X * 8, Y * 8, 8, 8, Col, BLEND_ALPHA50
                    If Idx <> 0 Then noBlocks = False
                End If
                
            Next X
        Next Y
        
        If noBlocks Then
            Set collisionTiles(A) = Tiles(A)
        Else
            Set collisionTiles(A) = Surface
        End If
        
    Next A
    
End Function
